#
# Copyright (C) 2020 Arm Mbed. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
#
"""Exposes a click command which prints information about environment variables of all child mbed packages."""
import click
import pdoc
from typing import Iterable


from mbed_devices.mbed_tools import env_variables as mbed_devices_env_variables


@click.command()
def cli() -> None:
    """Prints information about environment variables of all child mbed packages."""
    click.echo(_build_output(mbed_devices_env_variables))


_OUTPUT_PREAMBLE = """These variables used by Mbed Tools can be set either directly via environment variables or
using a `.env` file containing the variable definitions as follows:

VARIABLE=value

Environment variables take precedence, meaning the values set in the file will be overriden
by any values previously set in your environment.

+---------------------------------------------------------------------------------+
| Do not upload `.env` files containing private tokens to version control! If you |
| use this package as a dependency of your project, please ensure to include the  |
| `.env` in your `.gitignore`.                                                    |
+---------------------------------------------------------------------------------+
"""


def _build_output(env_variables: Iterable[pdoc.Variable]) -> str:
    variables_outputs = [f"{v.name}\n\n{_tab_prefix(v.docstring)}" for v in env_variables]
    variables_output = "\n\n".join(variables_outputs)
    return f"{_OUTPUT_PREAMBLE}\n\n{variables_output}"


def _tab_prefix(mutiline_text: str) -> str:
    return "\n".join([f"\t{line}" for line in mutiline_text.split("\n")])



#
# Copyright (C) 2020 Arm Mbed. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
#
"""Integration point with all sub-packages."""
import logging
from pkg_resources import working_set

from typing import Union, Any
import click

from mbed_tools_lib.logging import set_log_level, MbedToolsHandler

from mbed_build.mbed_tools import cli as mbed_build_export_cli
from mbed_devices.mbed_tools import cli as mbed_devices_cli
from mbed_tools._internal.env_cli import cli as env_cli


CONTEXT_SETTINGS = dict(help_option_names=["-h", "--help"])
LOGGER = logging.getLogger(__name__)


class GroupWithExceptionHandling(click.Group):
    """A click.Group which handles ToolsErrors and logging."""

    def invoke(self, context: click.Context) -> None:
        """Invoke the command group.

        Args:
            context: The current click context.
        """
        # Use the context manager to ensure tools exceptions (expected behaviour) are shown as messages to the user,
        # but all other exceptions (unexpected behaviour) are shown as errors.
        with MbedToolsHandler(LOGGER, context.params["traceback"]):
            super().invoke(context)


def print_version(context: click.Context, param: Union[click.Option, click.Parameter], value: bool) -> Any:
    """A click callback which prints the versions of all Mbed Python packages."""
    if not value or context.resilient_parsing:
        return

    # Get a list of Mbed Python packages
    mbed_packages = {d.project_name: d.version for d in working_set if d.project_name.startswith("mbed-")}

    # Show this package at the top of the list
    top_level_version_string = f"mbed-tools: {mbed_packages.pop('mbed-tools', '<unknown>')}"
    click.echo(top_level_version_string)
    click.echo("-" * len(top_level_version_string))

    # Show the remainder of the packages in alphabetical order
    for project_name, version in sorted(mbed_packages.items()):
        click.echo(f"{project_name}: {version}")

    context.exit()


@click.group(cls=GroupWithExceptionHandling, context_settings=CONTEXT_SETTINGS)
@click.option(
    "--version",
    is_flag=True,
    callback=print_version,
    expose_value=False,
    is_eager=True,
    help="Display versions of all Mbed Tools packages.",
)
@click.option(
    "-v",
    "--verbose",
    default=0,
    count=True,
    help="Set the verbosity level, enter multiple times to increase verbosity.",
)
@click.option("-t", "--traceback", is_flag=True, show_default=True, help="Show a traceback when an error is raised.")
def cli(verbose: int, traceback: bool) -> None:
    """Command line tool for interacting with Mbed OS."""
    set_log_level(verbose)


cli.add_command(mbed_build_export_cli, "export-keys")
cli.add_command(mbed_devices_cli, "devices")
cli.add_command(env_cli, "env")



#
# Copyright (C) 2020 Arm Mbed. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
#
"""Package definition for PyPI."""
import os

from setuptools import setup

PROJECT_SLUG = "mbed-tools"
SOURCE_DIR = "mbed_tools"
__version__ = None

repository_dir = os.path.dirname(__file__)

# Read package version, this will set the variable `__version__` to the current version.
with open(os.path.join(repository_dir, SOURCE_DIR, "_version.py"), encoding="utf8") as fh:
    exec(fh.read())

# Use readme needed as long description in PyPI
with open(os.path.join(repository_dir, "README.md"), encoding="utf8") as fh:
    long_description = fh.read()

setup(
    author="Mbed team",
    author_email="support@mbed.com",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: Apache Software License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Topic :: Software Development :: Build Tools",
        "Topic :: Software Development :: Embedded Systems",
    ],
    description="Command line interface for Mbed OS.",
    keywords="Arm Mbed OS MbedOS cli command line tools",
    include_package_data=True,
    install_requires=["python-dotenv", "Click==7.0", "mbed-build", "mbed-devices", "pdoc3", "mbed-tools-lib~=1.2"],
    license="Apache 2.0",
    long_description_content_type="text/markdown",
    long_description=long_description,
    name=PROJECT_SLUG,
    packages=[SOURCE_DIR],
    python_requires=">=3.6,<4",
    url=f"https://github.com/ARMmbed/{PROJECT_SLUG}",
    version=__version__,
    entry_points={
        "console_scripts": [
            f"mbedtools={SOURCE_DIR}.cli:cli",
            f"mbed-tools={SOURCE_DIR}.cli:cli",
            f"mbed_tools={SOURCE_DIR}.cli:cli",
        ]
    },
)



#
# Copyright (C) 2020 Arm Mbed. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
#
"""The version number is based on semantic versioning.

References
- https://semver.org/
- https://www.python.org/dev/peps/pep-0440/

This file is autogenerated, do not modify by hand.
"""
__version__ = "2.0.0-beta.22"
COMMIT = "aec360c5529e8f705b5ff0a3a359ef7053a66fec"
MAJOR = 2
MINOR = 0
PATCH = 0
